
/* ========= */
/* bm_icon.h */
/* ========= */

#pragma once

#define ICO_INFO 125

#define ICO_STOPOUI (128 + 76)
#define ICO_STOPNON (128 + 77)
#define ICO_CHAR_LITW (128 + 78)

#define ICO_SOLMASK (128 + 124)
#define ICO_SOLDALLE3 (128 + 125)
#define ICO_SOLDALLE4 (128 + 126)
#define ICO_SOLDALLE5 (128 + 127)

#define ICO_SOL (256 + 0)
#define ICO_SOLCARRE (256 + 0)
#define ICO_SOLPAVE (256 + 1)
#define ICO_SOLDALLE1 (256 + 2)
#define ICO_SOLDALLE2 (256 + 3)
#define ICO_SOLELECTRO (256 + 4)
#define ICO_SOLELECTRO_D (256 + 11)
#define ICO_SOLOBJET (256 + 12)
#define ICO_SOLOBJET_D (256 + 15)
#define ICO_SOLMAX (256 + 25) /* sol si <= */

#define ICO_INVINCIBLE (256 + 16)
#define ICO_TANKBAS (256 + 17)
#define ICO_CAISSEGBAS (256 + 18)
#define ICO_CAISSEOBAS (256 + 19)
#define ICO_CAISSEVBAS (256 + 20)
#define ICO_TROUBOUCHE (256 + 21)
#define ICO_CAISSEBAS (256 + 22)
#define ICO_CELARROWS (256 + 23)
#define ICO_BAISSEBAS (256 + 24)
#define ICO_GLISSE (256 + 26)
#define ICO_CROIX (256 + 27)
#define ICO_DEPART (256 + 28)
#define ICO_ARRIVEE (256 + 63)
#define ICO_ARRIVEEPRIS (256 + 126)
#define ICO_ARRIVEEBOUM (256 + 127)
#define ICO_ARRIVEEVIDE (256 + 25)
#define ICO_TROU (256 + 29)
#define ICO_BORDG (256 + 30)
#define ICO_BORDD (256 + 31)

#define ICO_ARROWS (256 + 46)
#define ICO_TELECOM (256 + 47)
#define ICO_CHAR_LIT (256 + 48)
#define ICO_CHAR_BIG (256 + 50)

#define ICO_BLOQUE (256 + 48) /* pas de passage si >= */

#define ICO_PLANTEBAS (256 + 51)
#define ICO_PLANTEBAS_D (256 + 53)
#define ICO_PLANTEHAUT (256 + 54)
#define ICO_PLANTEHAUT_D (256 + 57)

#define ICO_TABLEVIDE (256 + 58)
#define ICO_TABLEBOIT (256 + 59)
#define ICO_LIVRE (256 + 60)
#define ICO_LUNETTES (256 + 61)
#define ICO_MAGIC (256 + 62)

#define ICO_MURHAUT (256 + 64)
#define ICO_MURHAUT_D (256 + 74)
#define ICO_MURBAS (256 + 80)
#define ICO_MURBAS_D (256 + 90)
#define ICO_BARRIERE (256 + 96)
#define ICO_BARRIERE_D (256 + 106)
#define MUR_NE 0
#define MUR_SE 1
#define MUR_SO 2
#define MUR_NO 3
#define MUR_NS 4
#define MUR_EO 5
#define MUR_SEO 6
#define MUR_ONS 7
#define MUR_NEO 8
#define MUR_ENS 9
#define MUR_NSEO 10

#define ICO_TECHNO1 (256 + 75)
#define ICO_TECHNO1_D (256 + 79)
#define ICO_TECHNO2 (256 + 91)
#define ICO_TECHNO2_D (256 + 95)
#define ICO_ELECTROBAS (256 + 107)
#define ICO_ELECTROBAS_D (256 + 109)
#define ICO_ELECTROHAUT (256 + 110)
#define ICO_ELECTROHAUT_D (256 + 111)
#define ICO_SENSUNI_S (256 + 112)
#define ICO_SENSUNI_E (256 + 113)
#define ICO_SENSUNI_N (256 + 114)
#define ICO_SENSUNI_O (256 + 115)
#define ICO_AIMANT (256 + 124)
#define ICO_TABLEPOISON (256 + 125)

#define ICO_OBSTACLE (256 + 128 + 0)
#define ICO_OBSTACLE_D (256 + 128 + 10)
#define ICO_TANK_E (256 + 128 + 11)
#define ICO_TANK_N (256 + 128 + 12)
#define ICO_TANK_O (256 + 128 + 13)
#define ICO_TANK_S (256 + 128 + 14)
#define ICO_JOUEUR (256 + 128 + 15)

#define ICO_PORTEF_EO (256 + 128 + 16)
#define ICO_PORTEF_NS (256 + 128 + 19)
#define ICO_PORTEO_EO (256 + 128 + 22)
#define ICO_PORTEO_NS (256 + 128 + 25)
#define ICO_CLE_A (256 + 128 + 28)
#define ICO_CLE_B (256 + 128 + 29)
#define ICO_CLE_C (256 + 128 + 30)
#define ICO_CLE_VIDE (256 + 128 + 31)

#define ICO_ACCEL_S (256 + 128 + 32)
#define ICO_ACCEL_E (256 + 128 + 33)
#define ICO_ACCEL_N (256 + 128 + 34)
#define ICO_ACCEL_O (256 + 128 + 35)

#define ICO_BAISSE (256 + 128 + 36)

#define ICO_ARROWUP (256 + 128 + 44)
#define ICO_ARROWDOWN (256 + 128 + 46)

#define ICO_BOIS1_NS (256 + 128 + 48)
#define ICO_BOIS2_NS (256 + 128 + 49)
#define ICO_BOIS3_NS (256 + 128 + 50)
#define ICO_BOIS1_EO (256 + 128 + 51)
#define ICO_BOIS2_EO (256 + 128 + 52)
#define ICO_BOIS3_EO (256 + 128 + 53)

#define ICO_UNSEUL (256 + 128 + 54)
#define ICO_CAISSE (256 + 128 + 62)
#define ICO_CAISSEV (256 + 128 + 63)
#define ICO_VITRE (256 + 128 + 64)
#define ICO_VITRE_D (256 + 128 + 74)
#define ICO_VITREC_NS (256 + 128 + 75)
#define ICO_VITREC_EO (256 + 128 + 77)
#define ICO_CAISSEO (256 + 128 + 79)

#define ICO_MEUBLE (256 + 128 + 80)
#define ICO_MEUBLE_D (256 + 128 + 91)
#define ICO_PIANO (256 + 128 + 89)
#define ICO_TELE (256 + 128 + 90)
#define ICO_TOURTE (256 + 128 + 91)
#define ICO_CAISSEG (256 + 128 + 95)

#define ICO_DETONATEUR_A (256 + 128 + 96)
#define ICO_DETONATEUR_B (256 + 128 + 97)
#define ICO_DETONATEUR_C (256 + 128 + 98)
#define ICO_BOMBE_A (256 + 128 + 102)
#define ICO_BOMBE_B (256 + 128 + 103)
#define ICO_BOMBE_C (256 + 128 + 104)
#define ICO_BOMBE_EX (256 + 128 + 105)

#define ICO_SAUVE (256 + 128 + 106)
#define ICO_PREND (256 + 128 + 107)
#define ICO_ATTENTE (256 + 128 + 108)
#define ICO_TANK_X (256 + 128 + 110)
#define ICO_TANK_EO (256 + 128 + 111)
#define ICO_CHAR_MID (256 + 128 + 112)

#define ICO_DEPARTOUV (256 + 128 + 123)
#define ICO_TANK_NS (256 + 128 + 127)

#define ICO_BUTTON_REL ((256 + 32) | UL)
#define ICO_BUTTON_PRESS ((256 + 32) | UR)
#define ICO_BUTTON_SPAL ((256 + 32) | DL)
#define ICO_BUTTON_PAUSE ((256 + 32) | DR)
#define ICO_BUTTON_BUILDSP (256 + 44)
#define ICO_BUTTON_BUILD (256 + 45)

#define ICO_BUTTON_ROND0 ((256 + 44) | DL)
#define ICO_BUTTON_ROND1 ((256 + 44) | DR)

#define ICO_OUTIL_BOIT ((256 + 33) | UL)
#define ICO_OUTIL_LIVRE ((256 + 33) | UR)
#define ICO_OUTIL_TRACKS ((256 + 33) | DL)
#define ICO_OUTIL_PLANTE ((256 + 33) | DR)
#define ICO_OUTIL_MUR ((256 + 34) | UL)
#define ICO_OUTIL_VISION ((256 + 34) | UR)
#define ICO_OUTIL_BARRIERE ((256 + 34) | DL)
#define ICO_OUTIL_TECHNO ((256 + 34) | DR)
#define ICO_OUTIL_ARRIVEE ((256 + 35) | UR)
#define ICO_OUTIL_DEPART ((256 + 35) | DL)
#define ICO_OUTIL_TROU ((256 + 35) | DR)
#define ICO_OUTIL_PLANTEBAS ((256 + 36) | UL)
#define ICO_OUTIL_SENSUNI ((256 + 36) | UR)
#define ICO_OUTIL_MAGIC ((256 + 36) | DL)
#define ICO_OUTIL_OBSTACLE ((256 + 36) | DR)
#define ICO_OUTIL_ELECTROBAS ((256 + 37) | UL)
#define ICO_OUTIL_ELECTRO ((256 + 37) | UR)
#define ICO_OUTIL_GLISSE ((256 + 37) | DL)
#define ICO_OUTIL_SOLCARRE ((256 + 37) | DR)
#define ICO_OUTIL_SOLPAVE ((256 + 38) | UL)
#define ICO_OUTIL_SOLDALLE1 ((256 + 38) | UR)
#define ICO_OUTIL_SOLELECTRO ((256 + 38) | DL)
#define ICO_OUTIL_SOLOBJET ((256 + 38) | DR)
#define ICO_OUTIL_PORTE ((256 + 39) | UL)
#define ICO_OUTIL_CLE ((256 + 39) | UR)
#define ICO_OUTIL_TRACKSBAR ((256 + 39) | DL)
#define ICO_OUTIL_AIMANT ((256 + 39) | DR)
#define ICO_OUTIL_ACCEL ((256 + 40) | UL)
#define ICO_OUTIL_BAISSE ((256 + 40) | UR)
#define ICO_OUTIL_JOUEUR ((256 + 40) | DL)
#define ICO_OUTIL_BOIS ((256 + 40) | DR)
#define ICO_OUTIL_VITRE ((256 + 41) | UL)
#define ICO_OUTIL_UNSEUL ((256 + 41) | UR)
#define ICO_OUTIL_CAISSE ((256 + 41) | DL)
#define ICO_OUTIL_MEUBLE ((256 + 41) | DR)
#define ICO_OUTIL_DETONATEUR ((256 + 42) | UL)
#define ICO_OUTIL_BOMBE ((256 + 42) | UR)
#define ICO_OUTIL_SOLDALLE2 ((256 + 42) | DL)
#define ICO_OUTIL_TANK ((256 + 42) | DR)
#define ICO_OUTIL_INVINCIBLE ((256 + 43) | UL)
#define ICO_OUTIL_SOLDALLE3 ((256 + 43) | UR)
#define ICO_OUTIL_SOLDALLE4 ((256 + 43) | DL)
#define ICO_OUTIL_SOLDALLE5 ((256 + 43) | DR)
